#!/usr/bin/perl

##############################################################################
#
#  ver-test.pl
#
#  Tool used for versional testing by scheme Old Master -> New Slave
#
#  Author:  Serge Kozlov skozlov@mysql.com
#  Date:    10/15/2007 
#
##############################################################################

use Cwd;
use strict;
use warnings;

our $wrapper_tmpl = 'wrapper.template.pl';

our $glob_cur_dir;

our $opt_builds_dir;
our $opt_except_dir = 'skjdnfksknfskfdsnndhhdff';
our $opt_master_version = '';
our $opt_slave_version = '';
our $opt_version_list;
our $opt_usage;

our $builds = {};
our $slave = {};
our $test_pairs = [];

our $mysqltest_argv = [];

our $slave_vars;

sub main ();
sub command_line_setup ();
sub usage ($);
sub test_pair($$);

######################################################################
#
#  Main program
#
######################################################################

main();
sub main ()
{
  my $line = "--------------------------------------------------------\n";
  print "MySQL versional testing.\n";
  print "Found MySQL versions:\n";
  command_line_setup();
  get_dir_list($opt_builds_dir);
  my $len_path = 0;
  my $len_ver = 0;
  foreach my $item (keys %$builds) 
  {
    $len_path = length($builds->{$item}->{'basedir'}) unless (length($builds->{$item}->{'basedir'}) lt $len_path); 
    $len_ver = length($builds->{$item}->{'version'}) unless (length($builds->{$item}->{'version'}) lt $len_ver); 
  }
  foreach my $item (sort keys %$builds) 
  {
    my $add_type = '';
    $add_type = 'slave' unless (!($item =~ /^$opt_slave_version$/));
    $add_type = 'master' unless (!($item =~ /^$opt_master_version$/));
    print sprintf "%-".$len_path."s\t%-".$len_ver."s\t\t%5s\n", $builds->{$item}->{'basedir'},$item,$add_type;
  }
  if ($opt_version_list) 
  {  
    exit(0);
  }
  $glob_cur_dir=  cwd();
  $slave_vars = get_mysqld_vars($opt_slave_version);
  create_pairs($opt_master_version,$opt_slave_version);
  print "Create wrappers for mysqld, ndb_mgmd, ndbd ";
  foreach my $pair (@$test_pairs)
  {
    create_wrapper('mysqld',$pair->{'master_version'},$pair->{'slave_version'});
    create_wrapper('ndb_mgmd',$pair->{'master_version'},$pair->{'slave_version'});
    create_wrapper('ndbd',$pair->{'master_version'},$pair->{'slave_version'});
    print ".";
  }
  print "\n";
  print "Install Slave Database\n";
  mkdir($builds->{$opt_slave_version}->{'basedir'}.'/mysql-test/var/slave-data');
  my $exe_install_db = $builds->{$opt_slave_version}->{'mysql_install_db'}.' --no-defaults --force --basedir='.$builds->{$opt_slave_version}->{'basedir'}.' --datadir='.$builds->{$opt_slave_version}->{'basedir'}.'/mysql-test/var/slave-data  > /dev/null';
  `$exe_install_db`;
  foreach my $pair (@$test_pairs)
  {
    print $line."Master: ".$pair->{'master_version'}.", Slave: ".$pair->{'slave_version'}."\n".$line;
    print run_mysqltest($pair->{'master_version'});
  }
  print $line;
  print "Remove wrappers for mysqld, ndb_mgmd, ndbd ";
  foreach my $pair (@$test_pairs)
  {
    remove_wrapper('mysqld',$pair->{'master_version'});
    remove_wrapper('ndbd',$pair->{'master_version'});
    remove_wrapper('ndb_mgmd',$pair->{'master_version'});
    print ".";
  }
  print "\n".$line;
  print "Finish.\n";
}


######################################################################
#
#  Read command line arguments
#
######################################################################

sub command_line_setup ()
{
  foreach my $arg (@ARGV) 
  {
    my @pair = split(/=/, $arg);
    if ($arg =~ /^--master-version=/) 
    {
      $opt_master_version = $pair[1];
    }
    elsif($arg =~ /^--slave-version=/)
    {
      $opt_slave_version = $pair[1];
    }
    elsif($arg =~ /^--version-list/)
    {
      $opt_version_list = 1;
    }
    elsif($arg =~ /^--builds-dir=/)
    {
      $opt_builds_dir = $pair[1];
    }
    elsif($arg =~ /^--except-dir=/)
    {
      $opt_except_dir = $pair[1];
    }
    else
    {
      push (@$mysqltest_argv,$arg);
    }
  }
}

sub get_dir_list ($)
{
  my $rdir = shift;
  opendir(DIR, $rdir) or die ("Cannot open directory $rdir");
  my @files = readdir(DIR);
  closedir(DIR);
  foreach my $item (@files) 
  {
    if (-e $rdir.'/'.$item && -d $rdir.'/'.$item && !($item =~/^\.{1,2}$/) && -d $rdir.'/'.$item.'/mysql-test' && !($item =~ /$opt_except_dir/i)) 
    {
      my $ref = {};
      $ref->{'basedir'} = $rdir.'/'.$item.'/';
      $ref->{'basedir'} =~ s/\/\//\//g;
      $ref->{'exe_mysqld'} = find_file($ref->{'basedir'},'mysqld');
      $ref->{'path_language'} = find_file($ref->{'basedir'},'language');
      $ref->{'path_charsets'} = find_file($ref->{'basedir'},'charsets');
      $ref->{'exe_ndb_mgmd'} = find_file($ref->{'basedir'},'ndb_mgmd');
      $ref->{'exe_ndbd'} = find_file($ref->{'basedir'},'ndbd');
      $ref->{'mysql_install_db'} = find_file($ref->{'basedir'},'mysql_install_db');
      $ref->{'version'} = get_version($ref);
      $builds->{get_version($ref)} = $ref;
    }
  }
}

sub find_file ($$)
{
  my $dir = shift;
  my $what = shift;
  my $rfile;
  my $dirs = {
    'mysqld' => {
      'files' => ['mysqld','mysqld-debug','mysqld-max','mysqld-nt','mysqld-max-nt'],
      'dirs'  => ['libexec','bin','sbin','sql','client','client-release'],    
      'type'  => 'file',
    },
    'mysql_install_db' => {
      'files' => ['mysql_install_db'],
      'dirs'  => ['bin','sql','client','client-release'],    
      'type'  => 'file',
    },
    'ndb_mgmd' => {
      'files' => ['ndb_mgmd'],
      'dirs'  => ['libexec','bin','sbin','ndb','storage/ndb','src/mgmsrv'],    
      'type'  => 'file',
    },
    'ndbd' => {
      'files' => ['ndbd'],
      'dirs'  => ['libexec','bin','sbin','ndb','storage/ndb','src/kernel'],    
      'type'  => 'file',
    },
    'language' => {
      'files' => ['english'],
      'dirs'  => ['share','share/mysql','sql/share'],    
      'type'  => 'dir',
     },
    'charsets' => {
      'files' => ['charsets'],
      'dirs'  => ['share','share/mysql','sql/share'],    
      'type'  => 'dir',
     },
  };
  my @paths = @{$dirs->{$what}{'dirs'}};
  my @files = @{$dirs->{$what}{'files'}};
  my $type = $dirs->{$what}{'type'};
  foreach my $path (@paths)
  {
    foreach my $file (@files) 
    {
      if ($type =~ /file/) 
      {
        if (-e $dir.$path.'/'.$file && !(-l $dir.$path.'/'.$file)) 
        {
          $rfile = $dir.$path.'/'.$file;
        }
        if ($rfile) { last };
      }
      elsif ($type =~ /dir/)
      {
        if (-e $dir.$path.'/'.$file && -d $dir.$path.'/'.$file) 
        {
          $rfile = $dir.$path.'/'.$file.'/';
        }
        if ($rfile) { last };
      }
    }
    if ($rfile) { last };
  }
  return $rfile;
}

sub get_version ($)
{
  my $info = shift;
  my $exe = $info->{'exe_mysqld'};
  my $new_exe = $exe;
  $new_exe =~ s/(.+)\/([a-z]+)/$1\/\_$2/;
  if (-e $new_exe) 
  {
    $exe = $new_exe;
  }
  $exe .= ' --basedir='.$info->{'basedir'}.' --language='.$info->{'path_language'}.' --character-sets-dir='.$info->{'path_charsets'};
  my $ver = `$exe --version`;
  $ver =~ s/^.+ Ver ([0-9.\-_a-zA-Z]+) .+$/$1/ig;
  $ver =~ s/\n//g;
  return $ver;
}

sub get_mysqld_vars ($)
{
  my $ver = shift;
  my $exe = $builds->{$ver}->{'exe_mysqld'};
  my $new_exe = $exe;
  $new_exe =~ s/(.+)\/([a-z]+)/$1\/\_$2/;
  if (-e $new_exe) 
  {
    $exe = $new_exe;
  }
  my $raw_output = `$exe --verbose --help`;
  my $vars;
  foreach my $line (split(/\n/,$raw_output)) 
  {
    if ($line =~ /\s(--[a-zA-Z\_\-0-9\#]+)/i) 
    {
      $vars .= $1.',';
    }
    elsif ($line =~ /\s(-[a-zA-Z0-9]{1})/i) 
    {
      $vars .= $1.',';
    }
  }
  if ((!($vars =~ /,--ndbcluster,/)) && $vars =~ /,--ndb-connectstring,/) 
  {
      $vars .= '--ndbcluster,';
  }
  return $vars;
}

sub create_pairs($$)
{
  my $master_ver = shift;
  my $slave_ver = shift;
  foreach my $item (keys %$builds) 
  {
    if (!($item =~ /$slave_ver/)) 
    {
      my $ref = {};
      $ref->{'master_version'} = $item;
      $ref->{'slave_version'} = $slave_ver;
      if ($master_ver =~ // || $master_ver =~ /$item/) 
      {
        push(@$test_pairs, $ref);
      }
    }
  }
}

sub create_wrapper ($$$)
{
  my $exe_type = shift;
  my $m_ver = shift;
  my $s_ver = shift;
  my $exe = $builds->{$m_ver}->{"exe_$exe_type"};
  my $new_exe = $exe;
  $new_exe =~ s/(.+)\/($exe_type)/$1\/\_$2/;
  open F, $wrapper_tmpl;
  my $content;
  while (<F>) 
  {
    $content .= $_;
  };
  close F;
  if (!(-e $new_exe)) 
  {
    system('mv', ($exe,$new_exe));
  }
  my $s_exe = $builds->{$s_ver}->{"exe_$exe_type"};
  $s_exe =~ s/(.+)\/([a-z]+)/$1\/\_$2/;
  if (!(-e $s_exe)) 
  {
    $s_exe = $builds->{$s_ver}->{"exe_$exe_type"};
  }
  my $data = "my \$exe = \"".$new_exe."\";\nmy \$slave_exe = \"".$s_exe."\";\n";
  $data .= "my \$slave_src_dir = \"".$builds->{$s_ver}->{'basedir'}."/mysql-test/var/slave-data\";\n";
  $data .= "my \$slave_dst_dir = \"".$builds->{$m_ver}->{'basedir'}."/mysql-test/var/\";\n";
  $data .= "my \@upd_args = (\n";
  $data .= "  \"--language=".$builds->{$s_ver}->{'path_language'}."\",\n  \"--character-sets-dir=".$builds->{$s_ver}->{'path_charsets'}."\"\n,";
  $data .= "  \"--basedir=".$builds->{$s_ver}->{'basedir'}."\",";
  $data .= ");\n";
  if ($exe_type =~/mysqld/) 
  {
    $data .= "my \$supported_options = \"".$slave_vars."\";\n";
  }
  else
  {
    $data .= "my \$supported_options;\n";
  }
  $content =~ s/\_DATA\_/$data/;
  open F, ">".$exe;
  print F $content;
  close F;
  chmod(0755,$exe);
}

sub run_mysqltest ($)
{
  my $ver = shift;
  my $mtr;
  my $mdir = $builds->{$ver}->{'basedir'}.'/mysql-test';
  chdir($mdir);
  if (-e './mysql-test-run.pl') 
  {
    $mtr = './mysql-test-run.pl';
  }
  else
  {
    $mtr = './mysql-test-run';
  }
  system($mtr, @$mysqltest_argv);
}

sub remove_wrapper ($$)
{
  my $exe_type = shift;
  my $ver = shift;
  my $exe = $builds->{$ver}->{"exe_$exe_type"};
  my $orig_exe = $exe;
  $orig_exe =~ s/(.+)\/($exe_type)/$1\/\_$2/;
  unlink($exe);
  system('mv', ($orig_exe, $exe));
}