#!/usr/bin/perl

############################################################
#
# Wrapper for mysqld/ndbd/ndb_mgmd binaries
# Purpose: Transparent switching between different binaries. 
#          Run another binary if slave arguments found.
#
# Author:  Serge Kozlov <skozlov@mysql.com>
# Date:    10/15/2007
#
############################################################

my @args = ();

_DATA_

my $is_slave;

if ($0 =~ /\/mysqld$/) 
{
  # Find a sign of mysqld slave
  foreach my $arg (@ARGV)
  { 
    if ($arg =~ /\/slave\.sock/)
    {
      $is_slave = 1;
      $exe = $slave_exe;
      last;
    }
  }
  if ($is_slave)
  {
    # copy db created by slave
    if (!(-e $slave_dst_dir.'/slave-data/.copied'))
    {
      system('rm',('-rf', $slave_dst_dir.'/slave-data/*'));
      system('cp',('-r', $slave_src_dir, $slave_dst_dir));
      system('touch',($slave_dst_dir.'/slave-data/.copied'));
    }
  }
}
elsif ($0 =~ /\/ndb_mgmd$/)
{
  # Find ndb slave port
  my $ndb_slave_port = $ENV{'NDBCLUSTER_PORT_SLAVE'};
  foreach my $arg (@ARGV)
  { 
    if ($arg =~ /--config-file=.+ndbcluster-$ndb_slave_port/)
    {
      $is_slave = 1;
      $exe = $slave_exe;
      last;
    }
  }
}
elsif ($0 =~ /\/ndbd$/)
{
  # Find ndb slave port
  my $ndb_slave_port = $ENV{'NDBCLUSTER_PORT_SLAVE'};
  foreach my $arg (@ARGV)
  { 
    if ($arg =~ /--ndb-connectstring=.+\:$ndb_slave_port/)
    {
      $is_slave = 1;
      $exe = $slave_exe;
      last;
    }
  }
}

if ($is_slave) 
{
  # Replace mysqld slave arguments defined in @upd_args
  for (my $idx = 0; $idx <= $#ARGV; $idx++)
  {
    my @pair = split(/=/,$ARGV[$idx]);
    my $item = $ARGV[$idx];   
    for (my $idx2 = 0; $idx2 <= $#upd_args; $idx2++)
    {
      my @upd_pair = split(/=/,$upd_args[$idx2]);
      if ($pair[0] =~ /^$upd_pair[0]$/)
      {
        $item = $upd_args[$idx2];
      } 
    }
    if ((!$supported_options) || ($supported_options =~ /$pair[0]\,/i)) 
    {
      push(@args, $item);
    }
  }
  if ($0 =~ /mysqld$/) 
  {
    open F, '>'.$slave_dst_dir.'/log.txt';
    print F join("\n", @args);
    print F "\n------------------------\n";
    print F join("\n", @ARGV);
    close F;
  }
}
else
{
  @args = @ARGV;
}

# Run binary
exec($exe, @args);
